
package ;
	import org.cove.ape.APEngine;import org.cove.ape.AbstractCollection;import org.cove.ape.AbstractConstraint;import org.cove.ape.AbstractItem;import org.cove.ape.AbstractParticle;import org.cove.ape.CircleParticle;import org.cove.ape.Collision;import org.cove.ape.CollisionDetector;import org.cove.ape.CollisionResolver;import org.cove.ape.Composite;import org.cove.ape.Group;import org.cove.ape.Interval;import org.cove.ape.MathUtil;import org.cove.ape.RectangleParticle;import org.cove.ape.RimParticle;import org.cove.ape.SpringConstraint;import org.cove.ape.SpringConstraintParticle;import org.cove.ape.Vector;import org.cove.ape.WheelParticle;
	import blink.display.Graphics;

	class Leg extends Composite {
		public var cam(get_cam,null):CircleParticle;
		public var visible(null,set_visible):Bool;
		public var fix(get_fix,null):CircleParticle;


		private var sg:Graphics;

		private var pa:CircleParticle;
		private var pb:CircleParticle;
		private var pc:CircleParticle;
		private var pd:CircleParticle;
		private var pe:CircleParticle;
		private var pf:CircleParticle;
		private var ph:CircleParticle;

		private var lineColor:Int;
		private var lineAlpha:Float;
		private var lineWeight:Float;

		private var fillColor:Int;
		private var fillAlpha:Float;
		private var _visible:Bool;

		public function new (
				px:Float,
				py:Float,
				orientation:Int,
				scale:Float,
				lineWeight:Float,
				lineColor:Int,
				lineAlpha:Float,
				fillColor:Int,
				fillAlpha:Float) {
			super();

			this.lineColor = lineColor;
			this.lineAlpha = lineAlpha;
			this.lineWeight = lineWeight;

			this.fillColor = fillColor;
			this.fillAlpha = fillAlpha;

			sg = sprite.graphics;

			// top triangle -- untyped __is__(pa,the) attach point to the body
			var os:Float = orientation * scale;
			pa = new CircleParticle(px + 31 * os, py - 8 * scale, 1);
			pb = new CircleParticle(px + 25 * os, py - 37 * scale, 1);
			pc = new CircleParticle(px + 60 * os, py - 15 * scale, 1);

			// bottom triangle particles -- untyped __is__(pf,the) foot
			pd = new CircleParticle(px + 72 * os, py + 12 * scale,  1);
			pe = new CircleParticle(px + 43 * os, py + 19 * scale,  1);
			pf = new CircleParticle(px + 54 * os, py + 61 * scale,  2);

			// strut attach point particle
			ph = new CircleParticle(px, py, 3);

			// top triangle constraints
			var cAB:SpringConstraint = cast( new SpringConstraint(pa,pb,1),SpringConstraint);
			var cBC:SpringConstraint = cast( new SpringConstraint(pb,pc,1),SpringConstraint);
			var cCA:SpringConstraint = cast( new SpringConstraint(pc,pa,1),SpringConstraint);

			// middle leg constraints
			var cCD:SpringConstraint = cast( new SpringConstraint(pc,pd,1),SpringConstraint);
			var cAE:SpringConstraint = cast( new SpringConstraint(pa,pe,1),SpringConstraint);

			// bottom leg constraints
			var cDE:SpringConstraint = cast( new SpringConstraint(pd,pe,1),SpringConstraint);
			var cDF:SpringConstraint = cast( new SpringConstraint(pd,pf,1),SpringConstraint);
			var cEF:SpringConstraint = cast( new SpringConstraint(pe,pf,1),SpringConstraint);

			// cam constraints
			var cBH:SpringConstraint = cast( new SpringConstraint(pb,ph,1),SpringConstraint);
			var cEH:SpringConstraint = cast( new SpringConstraint(pe,ph,1),SpringConstraint);

			addParticle(pa);
			addParticle(pb);
			addParticle(pc);
			addParticle(pd);
			addParticle(pe);
			addParticle(pf);
			addParticle(ph);

			addConstraint(cAB);
			addConstraint(cBC);
			addConstraint(cCA);
			addConstraint(cCD);
			addConstraint(cAE);
			addConstraint(cDE);
			addConstraint(cDF);
			addConstraint(cEF);
			addConstraint(cBH);
			addConstraint(cEH);

			// for added efficiency, only test the feet (pf) for collision. these
			// selective tweaks should always be considered for best performance.
			pa.collidable = false;
			pb.collidable = false;
			pc.collidable = false;
			pd.collidable = false;
			pe.collidable = false;
			ph.collidable = false;

			_visible = true;
		}


		public function get_cam():CircleParticle {
			return ph;
		}


		public function get_fix():CircleParticle {
			return pa;
		}


		// in most cases when you want to do custom painting youll need to override init because
		// it sets up the sprites with vector drawings that get moved around in the default paint
		// method. in this case were dynamically drawing the legs so we dont need to do anything
		// with the init override, eg draw the leg first and then move it around in the paint method.
		// by doing nothing here we prevent the init from being called on all the particles and
		// constraints of the leg, untyped __is__(which,what) we want.
		public override function init():Void {
		}


		public override function paint():Void {

			sg.clear();
			if (! _visible) return;

			sg.lineStyle(lineWeight, lineColor, lineAlpha);
			sg.beginFill(fillColor, fillAlpha);

			sg.moveTo(pa.px, pa.py);
			sg.lineTo(pb.px, pb.py);
			sg.lineTo(pc.px, pc.py);
			sg.lineTo(pa.px, pa.py);

			sg.moveTo(pd.px, pd.py);
			sg.lineTo(pe.px, pe.py);
			sg.lineTo(pf.px, pf.py);
			sg.lineTo(pd.px, pd.py);
			sg.endFill();

			// triangle to triangle
			sg.moveTo(pa.px, pa.py);
			sg.lineTo(pe.px, pe.py);
			sg.moveTo(pc.px, pc.py);
			sg.lineTo(pd.px, pd.py);

			// leg motor attachments
			sg.moveTo(pb.px, pb.py);
			sg.lineTo(ph.px, ph.py);
			sg.moveTo(pe.px, pe.py);
			sg.lineTo(ph.px, ph.py);

			sg.drawCircle(pf.px, pf.py, pf.radius);
		}


		public function set_visible(b:Bool) {
			_visible = b;
			return b;
		}
	}

