package ;
	import org.cove.ape.APEngine;import org.cove.ape.AbstractCollection;import org.cove.ape.AbstractConstraint;import org.cove.ape.AbstractItem;import org.cove.ape.AbstractParticle;import org.cove.ape.CircleParticle;import org.cove.ape.Collision;import org.cove.ape.CollisionDetector;import org.cove.ape.CollisionResolver;import org.cove.ape.Composite;import org.cove.ape.Group;import org.cove.ape.Interval;import org.cove.ape.MathUtil;import org.cove.ape.RectangleParticle;import org.cove.ape.RimParticle;import org.cove.ape.SpringConstraint;import org.cove.ape.SpringConstraintParticle;import org.cove.ape.Vector;import org.cove.ape.WheelParticle;
	import blink.display.Graphics;

	class Motor extends Composite {
		public var power(get_power,set_power):Float;
		public var rimA(get_rimA,null):AbstractParticle;
		public var rimB(get_rimB,null):AbstractParticle;
		public var rimC(get_rimC,null):AbstractParticle;


		private static var ONE_THIRD:Float = (Math.PI * 2) / 3;

		private var wheel:WheelParticle;
		private var radius:Float;
		private var _rimA:CircleParticle;
		private var _rimB:CircleParticle;
		private var _rimC:CircleParticle;
		private var sg:Graphics;
		private var color:Int;

		public function new(attach:AbstractParticle, radius:Float, color:Int) {
			super();

			sg = sprite.graphics;

			wheel = new WheelParticle(attach.px, attach.py - .01, radius);
			wheel.setStyle(0,0xFF0000,0, 0xFF0000,0.5);
			var axle:SpringConstraint = cast( new SpringConstraint(wheel, attach),SpringConstraint);

			_rimA = new CircleParticle(0,0,2, true);
			_rimB = new CircleParticle(0,0,2, true);
			_rimC = new CircleParticle(0,0,2, true);

			wheel.collidable = false;
			_rimA.collidable = false;
			_rimB.collidable = false;
			_rimC.collidable = false;

			addParticle(_rimA);
			addParticle(_rimB);
			addParticle(_rimC);
			addParticle(wheel);
			addConstraint(axle);

			this.color = color;
			this.radius = radius;

			// run it once to make sure the rim particles are in the right place
			run();
		}


		public function set_power(p:Float) {
			wheel.speed = p;
			return p;
		}


		public function get_power():Float {
			return wheel.speed;
		}


		public function get_rimA():CircleParticle {
			return _rimA;
		}


		public function get_rimB():CircleParticle {
			return _rimB;
		}


		public function get_rimC():CircleParticle {
			return _rimC;
		}


		public function run():Void {

			// align the rim particle based on the wheel rotation
			var theta:Float = wheel.radian;
			_rimA.px = -radius * Math.sin(theta) + wheel.px;
			_rimA.py =  radius * Math.cos(theta) + wheel.py;

			theta += ONE_THIRD;
			_rimB.px = -radius * Math.sin(theta) + wheel.px;
			_rimB.py =  radius * Math.cos(theta) + wheel.py;

			theta += ONE_THIRD;
			_rimC.px = -radius * Math.sin(theta) + wheel.px;
			_rimC.py =  radius * Math.cos(theta) + wheel.py;
		}


		// doing some custom painting here. contrast this with the custom painting of the
		// legs. in this case we draw the shape in the init method, and then just move/rotate
		// it in the paint method. one important thing here - the initial drawing happens in
		// object space (i.e., x = 0, y = 0) not world space. Another option would be to
		// just draw everything dynamically using the wheel and rim point locations.
		public override function init():Void {

			sg.clear();
			sg.lineStyle(0, color, 1);
			sg.beginFill(color);
			sg.drawCircle(0, 0, 3);
			sg.endFill();

			var theta:Float = 0;
			var cx:Float = -radius * Math.sin(theta);
			var cy:Float =  radius * Math.cos(theta);
			sg.moveTo(0,0);
			sg.lineTo(cx,cy);
			sg.drawCircle(cx, cy, 2);

			theta += ONE_THIRD;
			cx = -radius * Math.sin(theta);
			cy =  radius * Math.cos(theta);
			sg.moveTo(0,0);
			sg.lineTo(cx,cy);
			sg.drawCircle(cx, cy, 2);

			theta += ONE_THIRD;
			cx = -radius * Math.sin(theta);
			cy =  radius * Math.cos(theta);
			sg.moveTo(0,0);
			sg.lineTo(cx,cy);
			sg.drawCircle(cx, cy, 2);
		}

		public override function paint():Void {
			sprite.x = wheel.px;
			sprite.y = wheel.py;
			sprite.rotation = wheel.angle;
		}
	}

